unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  LCLIntf, LCLType, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure ImageDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ImageDragDrop(Sender, Source: TObject; X, Y: Integer);
  public
    ImageArray: Array of TImage;
    von, nach: Int32;
  end;

  { TMyDragObject }

  TMyDragObject = class(TDragControlObject)
  private
    FDragImages: TDragImageList;
  protected
    function GetDragImages: TDragImageList; override;
  public
    constructor Create(AControl: TControl); override;
    destructor Destroy; override;
  end;

var
  Form1: TForm1;

implementation

{ TMyDragObject }

function TMyDragObject.GetDragImages: TDragImageList;
begin
//  Result:=inherited GetDragImages;
  Result := FDragImages;
end;

constructor TMyDragObject.Create(AControl: TControl);
var
  Bitmap: TBitmap;
  i, xmin, xmax, ymin, ymax: Int32;
  mp: TPoint;
begin
  inherited Create(AControl);
  FDragImages := TDragImageList.Create(AControl);
  AlwaysShowDragImages := True;

  xmin:=Screen.Width;
  xmax:=0;
  ymin:=Screen.Height;
  ymax:=0;
  for i:=0 to High(Form1.ImageArray) do
    if Form1.ImageArray[i].Tag = 1 then begin
      if xmin > Form1.ImageArray[i].Left then xmin := Form1.ImageArray[i].Left;
      if xmax < Form1.ImageArray[i].Left then xmax := Form1.ImageArray[i].Left;
      if ymin > Form1.ImageArray[i].Top then ymin := Form1.ImageArray[i].Top;
      if ymax < Form1.ImageArray[i].Top then ymax := Form1.ImageArray[i].Top;
    end;

  Bitmap := TBitmap.Create;
  Bitmap.Width := xmax - xmin + Form1.ImageArray[0].Width;
  Bitmap.Height := ymax - ymin + Form1.ImageArray[0].Height;
  Bitmap.Canvas.Clear;

  for i:=0 to High(Form1.ImageArray) do
    if Form1.ImageArray[i].Tag = 1 then
      Bitmap.Canvas.Draw(
        Form1.ImageArray[i].Left - xmin,
        Form1.ImageArray[i].Top - ymin,
        Form1.ImageArray[i].Picture.Bitmap);

  FDragImages.Width := Bitmap.Width;
  FDragImages.Height := Bitmap.Height;
  FDragImages.AddMasked(Bitmap, $000000);

  GetCursorPos(mp);
  mp.x -= xmin;
  mp.y -= ymin;
  FDragImages.DragHotspot := Form1.ScreenToClient(mp);

  Bitmap.Free;
end;

destructor TMyDragObject.Destroy;
begin
  FDragImages.Free;
  inherited Destroy;
end;

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  ScrBitmap: TBitmap;
  ScreenDC: HDC;
  i, ix, iy: Int32;
begin
  ScrBitmap := TBitmap.Create;
  ScreenDC := GetDC(0);
  ScrBitmap.LoadFromDevice(ScreenDC);
  ReleaseDC(0, ScreenDC);

  SetLength(ImageArray, 100);
  for i:=0 to High(ImageArray) do begin
    ImageArray[i]:=TImage.Create(Self);
    ImageArray[i].Parent:=Self;
    ImageArray[i].Width:=Screen.Width div 16;
    ImageArray[i].Height:=Screen.Height div 16;
    ImageArray[i].Canvas.Clear;

    ix:=Random(ScrBitmap.Width * 15 div 16);
    iy:=Random(ScrBitmap.Height * 15 div 16);

    ImageArray[i].Picture.Bitmap.Canvas.CopyRect(
      Rect(0, 0, ImageArray[i].Width, ImageArray[i].Height),
      ScrBitmap.Canvas,
      Rect(ix, iy, ix + ImageArray[i].Width, iy + ImageArray[i].Height));

    ImageArray[i].Left:=i mod 10 * Screen.Width div 15 + Screen.Width div 32;
    ImageArray[i].Top:=i div 10 * Screen.Height div 15 + Screen.Height div 32;
 //   ImageArray[i].DragMode:=dmAutomatic;

    ImageArray[i].OnMouseDown:=@ImageMouseDown;
    ImageArray[i].OnStartDrag:=@ImageStartDrag;
    ImageArray[i].OnDragOver:=@ImageDragOver;
    ImageArray[i].OnDragDrop:=@ImageDragDrop;
  end;

  Width:=Screen.Width * 16 div 22;
  Height:=Screen.Height * 16 div 22;
  Position:=poScreenCenter;

  ScrBitmap.Free;
  Doublebuffered:=True;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  i: Int32;
begin
  Canvas.Pen.Color:=$0000FF;
  Canvas.Pen.Width:=2;

  for i:=0 to ComponentCount - 1 do
    if Components[i] is TImage then
      if Components[i].Tag = 1 then
        Canvas.Rectangle(
          TImage(Components[i]).Left - 1,
          TImage(Components[i]).Top - 1,
          TImage(Components[i]).Left + TImage(Components[i]).Width + 2,
          TImage(Components[i]).Top + TImage(Components[i]).Height + 2);
end;

procedure TForm1.ImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i: Int32;
  b: Boolean;
begin
  if ssShift in Shift then
    TImage(Sender).Tag:=1
  else begin
    for i:=0 to High(ImageArray) do
      ImageArray[i].Tag:=0;
    TImage(Sender).Tag:=1;
  end;

  for i:=0 to High(ImageArray) do
    if ImageArray[i] = Sender then von:=i;

  b:=False;
  for i:=0 to High(ImageArray) do
    if ImageArray[i].Tag = 1 then b:=True;
  if b then TImage(Sender).BeginDrag(False, 4);

  Repaint;
end;

procedure TForm1.ImageStartDrag(Sender: TObject; var DragObject: TDragObject);
begin
  DragObject := TMyDragObject.Create(Sender as TControl);
end;

procedure TForm1.ImageDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
var
  Control: TControl;
begin
  if Source is TControl then
    Control := Source as TControl
  else
  if Source is TDragControlObject then
    Control := (Source as TDragControlObject).Control
  else
    Control := nil;
  Accept := Control is TImage;
end;

procedure TForm1.ImageDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  i: Int32;
begin
  for i:=0 to High(ImageArray) do
    if ImageArray[i] = Sender then nach:=i;

  if nach <> von then
    for i:=0 to High(ImageArray) do
      if (ImageArray[i].Tag=1)
        and (i - von + nach >= 0)
        and (i - von + nach <= High(ImageArray))
        then ImageArray[i - von + nach].Canvas.Draw(0, 0, ImageArray[i].Picture.Bitmap);
end;

end.

